/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sample.jms;

import java.io.IOException;
import java.util.Date;
import javax.jms.MessageConsumer;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 *
 * @author ricky
 */
public class JmsServlet extends HttpServlet {

    public static final String FACTORY_NAME = "java:/comp/env/jms/sampleFactory";
    public static final String QUEUE_NAME = "java:/comp/env/jms/sampleQueue";
    
    public static final String JMS_USERNAME = "admin";
    public static final String JMS_PASSWORD = "admin";
    
    public static final String PARAMETER_OP = "op";
    public static final String PARAMETER_MESSAGE = "message";
    
    public static final String ATTRIBUTE_HISTORY = "history";
    
    public static final String SEND_OP = "send";
    public static final String RECEIVE_OP = "receive";
    public static final String RESET_OP = "reset";
    
    public static final String DEFAULT_MESSAGE = "(empty message)";
    
    public static final String INDEX_JSP = "/index.jsp";
    
    public static final long DELAY = 2000;
    
    /**
     * Initial context for JNDI
     */
    InitialContext initialContext = null;

    /**
     * Creates the initial context 
     * @param config The Servlet config
     * @throws ServletException Some error
     */
    @Override
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        try {
            initialContext = new InitialContext();
        } catch (Exception e) {
            throw new ServletException("Error getting initial context", e);
        }
    }

    /**
     * Destroy the initial context
     */
    @Override
    public void destroy() {
        super.destroy();
        if (initialContext != null) {
            try {
                initialContext.close();
            } catch (Exception e) {
            }
        }
        initialContext = null;
    }

    /** 
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        doPost(request, response);
    }
    
    /**
     * Send to the Queue the message
     * @param txt The text message to send to the queue
     * @throws ServletException Some error
     */
    private void doSend(String txt) throws ServletException {
        QueueConnection connection = null;
        QueueSession queueSession = null;
        try {
            System.out.println("Getting connection factory from JNDI...");
            QueueConnectionFactory connFactoryObj = (QueueConnectionFactory) initialContext.lookup(FACTORY_NAME);
            System.out.println("Creating connection...");
            connection = connFactoryObj.createQueueConnection(JMS_USERNAME, JMS_PASSWORD);
            queueSession = connection.createQueueSession(true, Session.SESSION_TRANSACTED);
            System.out.println("Getting queue from JNDI...");
            Queue queue = (Queue) initialContext.lookup(QUEUE_NAME);
            System.out.println("Creating the session");
            QueueSender queueSender = queueSession.createSender(queue);
            TextMessage message = queueSession.createTextMessage();
            message.setText(txt);
            System.out.println("Sending the message: " + message);
            queueSender.send(message);
            queueSession.commit();
        } catch (Exception e) {
            try {queueSession.rollback();} catch(Exception ex) {}
            e.printStackTrace();
            throw new ServletException(e);
        } finally {
            if (queueSession != null) {
                try {queueSession.close();} catch(Exception e) {}
            }
            if (connection != null) {
                try {connection.close();} catch(Exception e) {}
            }
        }
    }
    
    /**
     * Creates a consumer and receive one message from the queue. It
     * waits DELAY milliseconds.
     * @return The text message or a custom message
     * @throws ServletException  Some error
     */
    private String doReceive() throws ServletException {
        QueueConnection connection = null;
        QueueSession queueSession = null;
        try {
            System.out.println("Getting connection factory from JNDI...");
            QueueConnectionFactory connFactoryObj = (QueueConnectionFactory) initialContext.lookup(FACTORY_NAME);
            System.out.println("Creating connection...");
            connection = connFactoryObj.createQueueConnection(JMS_USERNAME, JMS_PASSWORD);
            queueSession = connection.createQueueSession(true, Session.SESSION_TRANSACTED);
            System.out.println("Getting queue from JNDI...");
            Queue queue = (Queue) initialContext.lookup(QUEUE_NAME);
            System.out.println("Creating the session");
            MessageConsumer queueConsumer = queueSession.createConsumer(queue);
            System.out.println("Receiving a message...");
            connection.start();
            TextMessage message = (TextMessage) queueConsumer.receive(DELAY);
            System.err.println("Received: " + message);
            queueSession.commit();
            if (message != null) {
                return message.getText();
            } else {
                return "No message receive in " + DELAY + " milliseconds";
            }
        } catch (Exception e) {
            try {queueSession.rollback();} catch(Exception ex) {}
            e.printStackTrace();
            throw new ServletException(e);
        } finally {
            if (queueSession != null) {
                try {queueSession.close();} catch(Exception e) {}
            }
            if (connection != null) {
                try {connection.close();} catch(Exception e) {}
            }
        }
    }

    /** 
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        HttpSession session = request.getSession(true);
        String history = (String) session.getAttribute(ATTRIBUTE_HISTORY);
        if (history == null) {
            history = "";
        }
        String op = request.getParameter(PARAMETER_OP);
        System.err.println(PARAMETER_OP + ":" + op);
        if (SEND_OP.equals(op)) {
            String message = request.getParameter(PARAMETER_MESSAGE);
            if (message == null || message.isEmpty()) {
                message = DEFAULT_MESSAGE;
            }
            doSend(message);
            history = history + "\n[" + new Date() + "] Send   : " + message;
        } else if (RECEIVE_OP.equals(op)) {
            String message = doReceive();
            history = history + "\n[" + new Date() + "] Receive: " + message;
        } else if (RESET_OP.equals(op)) {
            history = "";
        }
        session.setAttribute(ATTRIBUTE_HISTORY, history);
        RequestDispatcher dispatcher = getServletContext().getRequestDispatcher(INDEX_JSP);
        dispatcher.forward(request, response);
    }

    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "JMS Servlet";
    }
}
